/*
This sketch illustrates ideas from the course "2D Animation and Interaction"
available at www.imaginary-institute.com, and my book "Processing for
Artists and Designers," published by AK Peters. For more of my Processing
programs, visit www.glassner.com or www.imaginary-institute.com
---
Andrew Glassner, version 2, April 2014
*/
Nomad[] NomadList;        // The nomads that are wandering
int NumberOfNomads = 300; // Tuned by hand; 300 looked good to me
int StepCount;            // how many steps they've taken
int MaxSteps;             // The number of steps this batch is allowed
float NoiseScale;         // How wiggly the trails are
float MouseYOffset = 30;  // because the browser isn't always aligned
boolean ClearBG = true;   // clear the background before drawing?
void setup() {
  size(800, 600);
  smooth();
  background(0);
}
// Create a circle of Nomads. Each one is randomly placed on
// the circle, and has some variation of the shared color.
void initNomads(float diskX, float diskY) {
  NomadList = new Nomad[NumberOfNomads];
  float diskR = random(25, 100);  //pick a radius for the circle of starting points
  colorMode(HSB);  // I want vibrant colors of different hues, so HSB is the right color space
    color baseClr = color(random(0,255), random(100,255), random(170,255));
  colorMode(RGB);
  // start off each nomad
  for (int i=0; i<NomadList.length; i++) {
    float angle = random(0, TWO_PI);
    color thisClr = wiggleColorHSB(baseClr, .1, .1, .1);
    NomadList[i] = new Nomad(diskX+(diskR*cos(angle)),
                             diskY+(diskR*sin(angle)),
                             thisClr,
                             1);
  }
  StepCount = 0;
  MaxSteps = int(random(200, 800)); // how many times to draw
  // low values of noise give long, thin tendrils. High values are twistier.
  NoiseScale = random(.0001, .03);
}
// given an HSB color, wiggle each of the values around a bit
color wiggleColorHSB(color c, float wh, float ws, float wb) {
  colorMode(HSB);
  float newHue = (hue(c)+(wh*255*random(-1, 1))) % 255;
  float newSat = constrain(saturation(c)+(ws*255*random(-1,1)), 0, 255);
  float newBrt = constrain(brightness(c)+(wb*255*random(-1,1)), 0, 255);
  color newClr = color(newHue, newSat, newBrt);
  colorMode(RGB);
  return(newClr);
}
void draw() {
  if (ClearBG) {
    background(0);
    ClearBG = false;
  }
  if (++StepCount > MaxSteps) {
    return;
  }
  for (int i=0; i<NomadList.length; i++) {
    NomadList[i].render();
    NomadList[i].update();
  }
}
void keyTyped() {
  if (key == 'c') ClearBG = true;
  if (key == 'y') MouseYOffset++;
  if (key == 'Y') MouseYOffset--;
}
void mousePressed() {
  initNomads(mouseX, mouseY-MouseYOffset);
}
//----------- Nomad class -------------
class Nomad {
  PVector velocity;     // speed and direction of movement
  PVector pos, oldPos;  // where we are now (and where we were), so we can draw a line
                        //    after each step, joining our old position and our new one
  color clr;            // our color
  float r, opacity;     // our radius (stroke weight) and opacity
  Nomad(float startX, float startY, color startClr, float startR) {
    pos = new PVector(startX, startY);  // start at the given point
    oldPos = pos.get();                 // remember this as the "old" position
    clr = startClr;                     // remember our color
    r = startR;                         // remember our radius (or weight)
    opacity = random(.25, .5);          // pick an opacity from 25% to 50%
    velocity = new PVector(random(.5, 1), random(.5, 1));  // pick a new movement direction
    if (random(100)>50) velocity.x *= -1;  // half the time, move left
    if (random(100)>50) velocity.y *= -1;  // half the time, move up
  }
  /* Each Nomad has a velocity and a position. On each frame, the
  position is updated by the velocity. The velocity changes on every
  frame, inheriting some change from the underlying noise. */
  void update() {
    // Get a new value for the velocity from the noise
    float vx = noise(pos.x*NoiseScale, pos.y*NoiseScale);
    float vy = noise((2*width)+(pos.x*NoiseScale), (2*height)+(pos.y*NoiseScale));
    // Noise values are from (0,1) but we want them from (-1,1)
    vx = lerp(-1, 1, vx);
    vy = lerp(-1, 1, vy);
    // Get some "blending factors" to control how the velocity updates
    float bx = lerp(.25, .75, noise(pos.y * NoiseScale));
    float by = lerp(.25, .75, noise(pos.x * NoiseScale));
    // Add some of the new values to the velocity, so the velocity changes, but slowly
    velocity.x += bx*vx;
    velocity.y += by*vy;
    // Normalize the velocity, save the old point, update the position
    velocity.normalize();
    oldPos = pos.get();
    pos.x += velocity.x;
    pos.y += velocity.y;
    // Finally, make the particle just ever so slightly less opaque as it ages
    opacity *= .995;
  }
  void render() {
    stroke(clr, opacity*255);
    strokeWeight(r);
    noFill();
    line(oldPos.x, oldPos.y, pos.x, pos.y);
  }
}
